package sinhalaocr;

import java.awt.*;
import java.awt.image.PixelGrabber;
import java.awt.image.*;
import java.util.*;
import javax.swing.*;
import java.awt.image.MemoryImageSource;
import java.io.*;
import java.util.ArrayList;

class DocumentRec {
  private int height;
  private int width;
  private int data[];
  private int lineCount;
  private int averageLineHeight;
  private int line_start_info[], line_end_info[];
  private Line lineArray[];

  public DocumentRec(String infile) {
    Image image = Toolkit.getDefaultToolkit().getImage(infile);
    try {
      PixelGrabber grabber = new PixelGrabber(image, 0, 0, -1, -1, false);
      if (grabber.grabPixels()) {
        this.height = grabber.getHeight();
        this.width = grabber.getWidth();
        this.data = (int[]) grabber.getPixels();
      }
    }
    catch (InterruptedException e) {

    }

  }

  public void setNewData(int cleanData[]) {
    this.data = cleanData;
  }

  public int getHeight() {
    return this.height;
  }

  public int getWidth() {
    return this.width;
  }

  public int[] getData() {
    return this.data;
  }

  public int getLineCount() {
    return this.lineCount;
  }

  public int getAverageLineHeight() {
    return this.averageLineHeight;
  }

  public Line getLine(int lineNo) {
    return this.lineArray[lineNo];
  }

  public void setData(int index, int value) {
    this.data[index] = (255 << 24 | value << 16 | value << 8 | value);
  }

  public void setLineArray() {
    this.lineArray = new Line[this.getLineCount()];
    int arrayLength = this.lineArray.length;
    for (int i = 0; i < arrayLength; i++)
      lineArray[i] = new Line(i, this.line_start_info[i], this.line_end_info[i],
                              this.getWidth());
  }

  public void binarize(int threshold) {
    int temp_data[] = this.getData();
    int h = this.getHeight();
    int w = this.getWidth();
    for (int i = 0; i < h; i++)
      for (int j = 0; j < w; j++) {
        int blue = temp_data[i * w + j] & 0xff;
        int green = (temp_data[i * w + j] >> 8) & 0xff;
        int red = (temp_data[i * w + j] >> 16) & 0xff;
        int value = (red + green + blue) / 3;
        if (value > threshold)
          value = 255;
        else
          value = 0;
        this.setData(i * w + j, value);
      }
  }

  public int[] horizontalProjection() {
    this.binarize(160);
    int temp_data[] = this.getData();
    int horizontalProjection[] = new int[height];
    int sum = 0;
    int h = this.getHeight();
    int w = this.getWidth();
    for (int i = 0; i < h; i++) {
      for (int j = 0; j < w; j++) {
        if ( ( (temp_data[i * w + j]) & 0xff) < 160)
          sum++;
      }
      horizontalProjection[i] = sum;
      sum = 0;
    }
    return horizontalProjection;
  }

  public int[] horizontalProjection(int imData[], int h, int w) {
    int horizontalProjection[] = new int[h];
    int sum = 0;
    for (int i = 0; i < h; i++) {
      for (int j = 0; j < w; j++) {
        if ( ( (imData[ (i * w) + j]) & 0xff) < 160)
          sum++;
      }
      horizontalProjection[i] = sum;
      sum = 0;
    }
    return horizontalProjection;
  }

  public int[] removeHP(int hpData[]) {
    for (int y = 0; y < hpData.length; y++) {
      if (hpData[y] < 30)
        hpData[y] = 0;
    }
    return hpData;
  }

  public int[] paraInfo() {
    int hp[] = this.horizontalProjection();
    this.line_start_info = new int[this.getLineCount()];
    this.line_end_info = new int[this.getLineCount()];
    int avgLineHeight = 0, line_start = 0, line_end = 0, no_of_line = 0,
        lineSt = 0, lineEd = 0, lineIndex = 0, paraNum = 0;
    int para[] = new int[hp.length];
    boolean non_zero = false;
    for (int i = 0; i < hp.length; i++) {
      if (hp[i] != 0) {
        if (!non_zero) {
          non_zero = true;
          line_start = i;
          this.line_start_info[lineSt++] = line_start;
        }
      }
      else {
        if (non_zero) {
          non_zero = false;
          line_end = i - 1;
          lineIndex++;
          // if((line_end - line_start)>20)
          // para[paraNum++]=lineIndex-1;
          //avgLineHeight = avgLineHeight + (line_end - line_start);
          this.line_end_info[lineEd++] = line_end;
        }
      }
    }
    int paraInfo[] = new int[lineIndex];

    for (int j = 0; j < paraInfo.length - 1; j++) {
      int dif = (this.line_start_info[j + 1] - this.line_end_info[j]);
      if (dif > 60)
        paraInfo[paraNum++] = j;
    }

    return paraInfo;

  }

  public int calculateLineCount() {
    int lineCount = 0;
    boolean non_zero = false;
    int hp[] = this.horizontalProjection();
    for (int i = 0; i < hp.length; i++) {
      if (hp[i] != 0) {
        if (non_zero == false) {
          non_zero = true;
        }
      }
      if (hp[i] == 0) {
        if (non_zero == true) {
          non_zero = false;
          lineCount++;
        }
      }
    }
    this.lineCount = lineCount;
    return lineCount;
  }

  public int averageLineHeight() {
    int hp[] = this.horizontalProjection();
    this.line_start_info = new int[this.getLineCount()];
    this.line_end_info = new int[this.getLineCount()];
    int avgLineHeight = 0, line_start = 0, line_end = 0, no_of_line = 0,
        lineSt = 0, lineEd = 0;
    boolean non_zero = false;
    for (int i = 0; i < hp.length; i++) {
      if (hp[i] != 0) {
        if (!non_zero) {
          non_zero = true;
          line_start = i;
          this.line_start_info[lineSt++] = line_start;
        }
      }
      else {
        if (non_zero) {
          non_zero = false;
          line_end = i - 1;
          avgLineHeight = avgLineHeight + (line_end - line_start);
          this.line_end_info[lineEd++] = line_end;
        }
      }
    }
    no_of_line = this.calculateLineCount();
    return ( (int) avgLineHeight) / (int) no_of_line;
  }

  public int[] getSubArray(int source[], int start, int end) {
    int temp[] = new int[ (end - start) + 1];
    for (int i = start; i <= end; i++)
      temp[i] = source[i];
    return temp;
  }

  public Char getCompactImage(int imageData[], int h, int w) {
    int compImage[] = this.horizontalProjection(imageData, h, w);
    int line_start = 0, line_end = 0;
    boolean non_zero = false;
    for (int i = 0; i < compImage.length; i++) {
      if (compImage[i] != 0) {
        if (!non_zero) {
          non_zero = true;
          line_start = i;
        }
      }
      else {
        if (non_zero) {
          non_zero = false;
          line_end = i - 1;
        }
      }
      if (non_zero && compImage[compImage.length - 1] != 0)
        line_end = compImage.length - 1;
    }
    int temp[] = new int[ (line_end - line_start + 1) * w];
    int t = 0;
    int lnEd = (line_end + 1) * (w);
    for (int u = (line_start * w); u < lnEd; u++) {
      temp[t] = imageData[u];
      t++;
    }
    return new Char(temp, (line_end - line_start) + 1, w);
  }

  public int getDifference(byte reference[], int input[]) {
    int sumSquared = 0;
    for (int i = 0; i < reference.length; i++) {
      sumSquared = sumSquared + (int) Math.pow(input[i] - (int) reference[i], 2);
    }
    return sumSquared;
  }

  public String knn(int inputVector[], int line, int ch, String font) {
    String letter = "OutNo.txt";
    int previous = 1000;

    try {
      String filePath = "";
      filePath = font.concat("\\");

      File dataFile = new File(filePath);
      File allFiles[] = dataFile.listFiles();
      for (int i = 0; i < allFiles.length; i++) {
        String path = allFiles[i].getPath();
        byte enter[] = {
            13, 10};
        RandomAccessFile currentFile = new RandomAccessFile(path, "r");
        byte vector[] = new byte[625];
        int d = 0;
        do {
          if ( (currentFile.read(vector) == -1) && (d > 15))
            break;
          int distance = getDifference(vector, inputVector);
          if (distance >= 150)
            break;
          d++;
          if (distance <= previous) {
            previous = distance;
            letter = allFiles[i].getName();
          }

        }
        while (true);
        currentFile.close();
      }
    }
    catch (Exception e) {
      System.out.println(e);
    }

    return letter;
  }

  public String[] unicodeValue(String in) {
    String uniVal[] = new String[5];
    if (in.equals("A")) {
      uniVal[0] = "0d85";
    }
    else if (in.equals("Adapilla")) {

      uniVal[0] = "0dd0";
    }
    else if (in.equals("Alapilla")) {

      uniVal[0] = "0dcf";
    }
    else if (in.equals("AlawithHal")) {

      uniVal[0] = "0dcf";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Anusvaraya")) {

      uniVal[0] = "0d82";
    }
    else if (in.equals("B")) {

      uniVal[0] = "0db6";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Ba")) {

      uniVal[0] = "0db6";
    }
    else if (in.equals("Bi")) {

      uniVal[0] = "0db6";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Bii")) {

      uniVal[0] = "0db6";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Bu")) {

      uniVal[0] = "0db6";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Buu")) {

      uniVal[0] = "0db6";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("Bra")) {

      uniVal[0] = "0db6";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";

    }
    else if (in.equals("Bri")) {

      uniVal[0] = "0db6";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
      uniVal[4] = "0dd2";
    }
    else if (in.equals("Brii")) {

      uniVal[0] = "0db6";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
      uniVal[4] = "0dd3";
    }

    else if (in.equals("BracketC")) {

      uniVal[0] = "0029";
    }
    else if (in.equals("BracketO")) {

      uniVal[0] = "0028";
    }

    else if (in.equals("Ch")) {

      uniVal[0] = "0da0";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Cha")) {

      uniVal[0] = "0da0";
    }
    else if (in.equals("Chi")) {

      uniVal[0] = "0da0";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Chii")) {

      uniVal[0] = "0da0";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Chu")) {

      uniVal[0] = "0da0";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Chuu")) {

      uniVal[0] = "0da0";
      uniVal[1] = "0dd6";
    }

    else if (in.equals("Com") || in.equals("Com2")) {

      uniVal[0] = "0dd9";
    }
    else if (in.equals("ClosingCom")) {
      uniVal[0] = "002c";
    }
    else if (in.equals("OpenCom")) {
      uniVal[0] = "0022";
    }

    else if (in.equals("D")) {

      uniVal[0] = "0da9";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Da")) {

      uniVal[0] = "0da9";
    }
    else if (in.equals("Dash")) {

      uniVal[0] = "002d";
    }
    else if (in.equals("Deeraga")) {

      uniVal[0] = "0dd1";
    }
    else if (in.equals("Dh")) {

      uniVal[0] = "0daf";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Dha")) {

      uniVal[0] = "0daf";
    }
    else if (in.equals("Dhaa")) {

      uniVal[0] = "0daf";
      uniVal[1] = "0dcf";
    }
    else if (in.equals("Dhe")) {

      uniVal[0] = "0daf";
      uniVal[1] = "0dd0";
    }
    else if (in.equals("Dhee")) {

      uniVal[0] = "0daf";
      uniVal[1] = "0dd1";
    }

    else if (in.equals("Dhi")) {

      uniVal[0] = "0daf";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Dhii")) {

      uniVal[0] = "0daf";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Dhra")) {

      uniVal[0] = "0daf";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";

    }
    else if (in.equals("Dhri")) {

      uniVal[0] = "0daf";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
      uniVal[4] = "0dd2";
    }

    else if (in.equals("Dhu")) {

      uniVal[0] = "0daf";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Dhuu")) {

      uniVal[0] = "0daf";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("Di")) {

      uniVal[0] = "0da9";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Dii")) {

      uniVal[0] = "0da9";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Dot")) {

      uniVal[0] = "002e";

    }
    else if (in.equals("Du")) {

      uniVal[0] = "0da9";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Duu")) {

      uniVal[0] = "0da9";
      uniVal[1] = "0dd6";
    }

    else if (in.equals("E")) {

      uniVal[0] = "0d91";
    }
    else if (in.equals("EE")) {

      uniVal[0] = "0d92";
    }
    else if (in.equals("Fa")) {

      uniVal[0] = "0dc6";
    }
    else if (in.equals("G")) {

      uniVal[0] = "0d9c";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Ga")) {

      uniVal[0] = "0d9c";
    }
    else if (in.equals("Gataya")) {

      uniVal[0] = "0dd8";
    }
    else if (in.equals("Gyanukittha")) {

      uniVal[0] = "0ddf";
    }
    else if (in.equals("Gi")) {

      uniVal[0] = "0d9c";
      uniVal[1] = "0dd2";

    }
    else if (in.equals("Gii")) {

      uniVal[0] = "0d9c";
      uniVal[1] = "0dd3";

    }
    else if (in.equals("Gn")) {

      uniVal[0] = "0da4";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Gna")) {

      uniVal[0] = "0da4";
    }

    else if (in.equals("Gra")) {

      uniVal[0] = "0d9c";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";

    }
    else if (in.equals("Gri")) {

      uniVal[0] = "0d9c";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
      uniVal[4] = "0dd2";

    }
    else if (in.equals("Grii")) {

      uniVal[0] = "0d9c";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
      uniVal[4] = "0dd3";

    }
    else if (in.equals("Gu")) {

      uniVal[0] = "0d9c";
      uniVal[1] = "0dd4";

    }
    else if (in.equals("Guu")) {

      uniVal[0] = "0d9c";
      uniVal[1] = "0dd6";

    }
    else if (in.equals("H")) {

      uniVal[0] = "0dc4";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Ha")) {

      uniVal[0] = "0dc4";
    }
    else if (in.equals("Hi")) {

      uniVal[0] = "0dc4";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Hii")) {

      uniVal[0] = "0dc4";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Hu")) {

      uniVal[0] = "0dc4";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Huu")) {

      uniVal[0] = "0dc4";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("I")) {

      uniVal[0] = "0d89";
    }
    else if (in.equals("II")) {

      uniVal[0] = "0d8a";
    }
    else if (in.equals("J")) {

      uniVal[0] = "0da2";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Ja")) {

      uniVal[0] = "0da2";
    }
    else if (in.equals("Ji")) {

      uniVal[0] = "0da2";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Jii")) {

      uniVal[0] = "0da2";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Jna")) {

      uniVal[0] = "0da5";

    }
    else if (in.equals("Jnaa")) {

      uniVal[0] = "0da5";
      uniVal[1] = "0dcf";
    }
    else if (in.equals("Jra")) {

      uniVal[0] = "0da2";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";

    }
    else if (in.equals("Ju")) {

      uniVal[0] = "0da2";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Juu")) {

      uniVal[0] = "0da2";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("K")) {

      uniVal[0] = "0d9a";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Ka")) {

      uniVal[0] = "0d9a";
    }
    else if (in.equals("Ki")) {

      uniVal[0] = "0d9a";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Kii")) {

      uniVal[0] = "0d9a";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Kra")) {

      uniVal[0] = "0d9a";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";

    }
    else if (in.equals("Kri")) {

      uniVal[0] = "0d9a";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
      uniVal[4] = "0dd2";

    }
    else if (in.equals("Krii")) {

      uniVal[0] = "0d9a";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
      uniVal[4] = "0dd3";

    }
    else if (in.equals("Ksha")) {

      uniVal[0] = "0d9a";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dc2";

    }
    else if (in.equals("Ku")) {

      uniVal[0] = "0d9a";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Kuu")) {

      uniVal[0] = "0d9a";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("L")) {

      uniVal[0] = "0dbd";
      uniVal[1] = "0dca";
    }
    else if (in.equals("La")) {

      uniVal[0] = "0dbd";
    }
    else if (in.equals("La-M")) {

      uniVal[0] = "0dc5";
    }
    else if (in.equals("Li")) {

      uniVal[0] = "0dbd";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Lii")) {

      uniVal[0] = "0dbd";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Li-M")) {

      uniVal[0] = "0dc5";
      uniVal[1] = "0dd2";
    }

    else if (in.equals("L-M")) {

      uniVal[0] = "0dc5";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Lu")) {

      uniVal[0] = "0dbd";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Luu")) {

      uniVal[0] = "0dbd";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("Lu-M")) {

      uniVal[0] = "0dc5";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("M")) {

      uniVal[0] = "0db8";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Ma")) {

      uniVal[0] = "0db8";
    }
    else if (in.equals("Mb")) {

      uniVal[0] = "0db9";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Mba")) {

      uniVal[0] = "0db9";
    }
    else if (in.equals("Mbi")) {

      uniVal[0] = "0db9";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Mbii")) {

      uniVal[0] = "0db9";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Mbu")) {

      uniVal[0] = "0db9";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Mbuu")) {

      uniVal[0] = "0db9";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("Mi")) {

      uniVal[0] = "0db8";
      uniVal[1] = "0dd2";

    }
    else if (in.equals("Mii")) {

      uniVal[0] = "0db8";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Mu")) {

      uniVal[0] = "0db8";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Muu")) {

      uniVal[0] = "0db8";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("Mra")) {

      uniVal[0] = "0dbb";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0db8";
    }
    else if (in.equals("Rma")) {

      uniVal[0] = "0db8";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";

    }

    else if (in.equals("N")) {

      uniVal[0] = "0db1";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Na")) {

      uniVal[0] = "0db1";
    }
    else if (in.equals("Na-M")) {

      uniVal[0] = "0dab";
    }
    else if (in.equals("N-Dha")) {

      uniVal[0] = "0db3";
    }
    else if (in.equals("N-Dhi")) {

      uniVal[0] = "0db3";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("N-Dhii")) {

      uniVal[0] = "0db3";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("N-Dhu")) {

      uniVal[0] = "0db3";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("N-Dhuu")) {

      uniVal[0] = "0db3";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("N-Da")) {

      uniVal[0] = "ac0d";
    }
    else if (in.equals("N-Du")) {

      uniVal[0] = "0da9";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("N-Di")) {

      uniVal[0] = "0da9";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("N-Dii")) {

      uniVal[0] = "0da9";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Ng")) {

      uniVal[0] = "0d9f";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Nga")) {

      uniVal[0] = "0d9f";
    }
    else if (in.equals("Ngi")) {

      uniVal[0] = "0d9f";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Ngii")) {

      uniVal[0] = "0d9f";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Ngu")) {

      uniVal[0] = "0d9f";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Ni")) {

      uniVal[0] = "0db1";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Nii")) {

      uniVal[0] = "0db1";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Ni-M")) {

      uniVal[0] = "0dab";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Nii-M")) {

      uniVal[0] = "0dab";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("N-M")) {

      uniVal[0] = "0dab";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Nu")) {

      uniVal[0] = "0db1";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Nuu")) {

      uniVal[0] = "0db1";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("Nu-M")) {

      uniVal[0] = "0dab";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Nuu-M")) {

      uniVal[0] = "0dab";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("O")) {

      uniVal[0] = "0d94";
    }
    else if (in.equals("OO")) {

      uniVal[0] = "0d95";
    }
    else if (in.equals("P")) {

      uniVal[0] = "0db4";
      uniVal[1] = "0dca";

    }
    else if (in.equals("Pa")) {

      uniVal[0] = "0db4";
    }
    else if (in.equals("Pi")) {

      uniVal[0] = "0db4";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Pii")) {

      uniVal[0] = "0db4";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Pra")) {

      uniVal[0] = "0db4";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
    }
    else if (in.equals("Pri")) {

      uniVal[0] = "0db4";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
      uniVal[4] = "0dd2";
    }
    else if (in.equals("Prii")) {

      uniVal[0] = "0db4";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
      uniVal[4] = "0dd3";
    }

    else if (in.equals("Prahal")) {

      uniVal[0] = "0db4";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
      uniVal[4] = "0dca";
    }
    else if (in.equals("Pu")) {

      uniVal[0] = "0db4";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Puu")) {

      uniVal[0] = "0db4";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("R")) {

      uniVal[0] = "0dbb";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Ra")) {

      uniVal[0] = "0dbb";

    }
    else if (in.equals("Re")) {

      uniVal[0] = "0dbb";
      uniVal[1] = "d00d";

    }
    else if (in.equals("Ree")) {

      uniVal[0] = "0dbb";
      uniVal[1] = "d10d";

    }
    else if (in.equals("Repha-Yansa")) {

      uniVal[0] = "0dbb";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dca";
      uniVal[4] = "200d";
      uniVal[5] = "0dba";
    }

    else if (in.equals("Ri")) {

      uniVal[0] = "0dbb";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Rii")) {

      uniVal[0] = "0dbb";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Ru")) {

      uniVal[0] = "0dbb";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Ruu")) {

      uniVal[0] = "0dbb";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("S")) {

      uniVal[0] = "0dc3";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Sa")) {

      uniVal[0] = "0dc3";
    }
    else if (in.equals("Sra")) {

      uniVal[0] = "0dc3";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
    }
    else if (in.equals("Sh")) {

      uniVal[0] = "0dc1";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Sh-M")) {

      uniVal[0] = "0dc2";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Sha")) {

      uniVal[0] = "0dc1";
    }
    else if (in.equals("Sha-M")) {

      uniVal[0] = "0dc2";
    }
    else if (in.equals("Shi")) {

      uniVal[0] = "0dc1";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Shii")) {

      uniVal[0] = "0dc1";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Shi-M")) {

      uniVal[0] = "0dc2";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Shii-M")) {

      uniVal[0] = "0dc2";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Shu-M")) {

      uniVal[0] = "0dc2";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Shuu-M")) {

      uniVal[0] = "0dc2";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("Shra")) {

      uniVal[0] = "0dc1";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
    }

    else if (in.equals("Shrahal")) {

      uniVal[0] = "0dc1";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
      uniVal[4] = "0dca";
    }
    else if (in.equals("Shri")) {

      uniVal[0] = "0dc1";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
      uniVal[4] = "0dd2";
    }
    else if (in.equals("Shrii")) {

      uniVal[0] = "0dc1";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Shu")) {

      uniVal[0] = "0dc1";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Shuu")) {

      uniVal[0] = "0dc1";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("Si")) {

      uniVal[0] = "0dc3";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Sii")) {

      uniVal[0] = "0dc3";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Su")) {

      uniVal[0] = "0dc3";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Suu")) {

      uniVal[0] = "0dc3";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("T")) {

      uniVal[0] = "0da7";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Ta")) {

      uniVal[0] = "0da7";
    }
    else if (in.equals("Th")) {

      uniVal[0] = "0dad";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Tha")) {

      uniVal[0] = "0dad";
    }
    else if (in.equals("Thi")) {

      uniVal[0] = "0dad";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Thii")) {

      uniVal[0] = "0dad";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Thra")) {

      uniVal[0] = "0dad";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
    }
    else if (in.equals("Thri")) {

      uniVal[0] = "0dad";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
      uniVal[4] = "0dd2";
    }
    else if (in.equals("Thrii")) {

      uniVal[0] = "0dad";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
      uniVal[4] = "0dd3";
    }
    else if (in.equals("Thu")) {

      uniVal[0] = "0dad";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Thuu")) {

      uniVal[0] = "0dad";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("Ti")) {

      uniVal[0] = "0da7";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Tii")) {

      uniVal[0] = "0da7";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Tu")) {

      uniVal[0] = "0da7";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Tuu")) {

      uniVal[0] = "0da7";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("Tra")) {

      uniVal[0] = "0da7";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
    }
    else if (in.equals("Tri")) {

      uniVal[0] = "0da7";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
      uniVal[4] = "0dd2";
    }
    else if (in.equals("Trii")) {

      uniVal[0] = "0da7";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
      uniVal[4] = "0dd3";
    }
    else if (in.equals("U")) {

      uniVal[0] = "0d8b";
    }
    else if (in.equals("V")) {

      uniVal[0] = "0dc0";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Va")) {

      uniVal[0] = "0dc0";
    }
    else if (in.equals("V-Ba")) {

      uniVal[0] = "0db7";

    }
    else if (in.equals("V-Bi")) {

      uniVal[0] = "0db7";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("V-Bii")) {

      uniVal[0] = "0db7";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("V-Bu")) {

      uniVal[0] = "0db7";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("V-Buu")) {

      uniVal[0] = "0db7";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("V-Cha")) {

      uniVal[0] = "0da1";
    }
    else if (in.equals("V-Da")) {

      uniVal[0] = "0daa";

    }
    else if (in.equals("V-Dha")) {

      uniVal[0] = "0db0";

    }
    else if (in.equals("V-Dhi")) {

      uniVal[0] = "0db0";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("V-Dhii")) {

      uniVal[0] = "0db0";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("V-Dhu")) {

      uniVal[0] = "0db0";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("V-Dhuu")) {

      uniVal[0] = "0db0";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("N-Di")) {

      uniVal[0] = "0db3";
      uniVal[1] = "0dd2";

    }
    else if (in.equals("N-Dii")) {

      uniVal[0] = "0db3";
      uniVal[1] = "0dd3";

    }
    else if (in.equals("N-Du")) {

      uniVal[0] = "0db3";
      uniVal[1] = "0dd4";

    }
    else if (in.equals("Question")) {

      uniVal[0] = "003f";

    }

    else if (in.equals("V-Ga")) {

      uniVal[0] = "0d9d";

    }
    else if (in.equals("V-Gra")) {

      uniVal[0] = "9d0d";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dbb";
    }
    else if (in.equals("V-Gu")) {

      uniVal[0] = "0d9d";
      uniVal[1] = "0dd4";

    }
    else if (in.equals("V-Guu")) {

      uniVal[0] = "0d9d";
      uniVal[1] = "0dd6";

    }

    else if (in.equals("Vi")) {

      uniVal[0] = "0dc0";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Vii")) {

      uniVal[0] = "0dc0";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("V-K")) {

      uniVal[0] = "0d9b";
      uniVal[1] = "0dca";
    }
    else if (in.equals("V-Ka")) {

      uniVal[0] = "0d9b";
    }
    else if (in.equals("V-Ki")) {

      uniVal[0] = "0d9b";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("V-Kii")) {

      uniVal[0] = "0d9b";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("V-Ku")) {

      uniVal[0] = "0d9b";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("V-Kuu")) {

      uniVal[0] = "0d9b";
      uniVal[1] = "0dd6";
    }

    else if (in.equals("V-Pa")) {

      uniVal[0] = "0db5";
    }
    else if (in.equals("V-Ta")) {

      uniVal[0] = "0dab";
    }
    else if (in.equals("V-Tha")) {

      uniVal[0] = "0dae";
    }
    else if (in.equals("V-Ti")) {

      uniVal[0] = "0dae";
      uniVal[1] = "0dd2";
    }

    else if (in.equals("V-Thi")) {

      uniVal[0] = "0dab";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Vu")) {

      uniVal[0] = "0dc0";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Vuu")) {

      uniVal[0] = "0dc0";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("Y")) {

      uniVal[0] = "0dba";
      uniVal[1] = "0dca";
    }
    else if (in.equals("Ya")) {

      uniVal[0] = "0dba";
    }
    else if (in.equals("Yansa")) {
      uniVal[0] = "0dca";
      uniVal[1] = "200d";
      uniVal[2] = "0dba";
    }
    else if (in.equals("Yansapapilla")) {
      uniVal[0] = "0dca";
      uniVal[1] = "200d";
      uniVal[2] = "0dba";
      uniVal[3] = "0dd4";
    }

    else if (in.equals("Yi")) {

      uniVal[0] = "0dba";
      uniVal[1] = "0dd2";
    }
    else if (in.equals("Yii")) {

      uniVal[0] = "0dba";
      uniVal[1] = "0dd3";
    }
    else if (in.equals("Yu")) {

      uniVal[0] = "0dba";
      uniVal[1] = "0dd4";
    }
    else if (in.equals("Yuu")) {

      uniVal[0] = "0dba";
      uniVal[1] = "0dd6";
    }
    else if (in.equals("No")) {
      uniVal[0] = "0058";
    }
    else if (in.equals("0")) {
      uniVal[0] = "0030";
    }
    else if (in.equals("1")) {
      uniVal[0] = "0031";
    }
    else if (in.equals("2")) {
      uniVal[0] = "0032";
    }
    else if (in.equals("3")) {
      uniVal[0] = "0033";
    }

    else if (in.equals("4")) {
      uniVal[0] = "0034";
    }

    else if (in.equals("5")) {
      uniVal[0] = "0035";
    }

    else if (in.equals("6")) {
      uniVal[0] = "0036";
    }

    else if (in.equals("7")) {
      uniVal[0] = "0037";
    }
    else if (in.equals("8")) {
      uniVal[0] = "0038";
    }
    else if (in.equals("9")) {
      uniVal[0] = "0039";
    }
    else if (in.equals("InvertO")) {
      uniVal[0] = "0022";
    }
    else if (in.equals("Add")) {
      uniVal[0] = "002b";
    }
    else if (in.equals("At")) {
      uniVal[0] = "0040";
    }
    else if (in.equals("Ast")) {
      uniVal[0] = "002a";
    }

    else if (in.equals("Div")) {
      uniVal[0] = "002f";
    }
    else if (in.equals("Exlam")) {
      uniVal[0] = "0021";
    }
    else if (in.equals("Grete")) {
      uniVal[0] = "003e";
    }
    else if (in.equals("Less")) {
      uniVal[0] = "003c";
    }
    else if (in.equals("Percent")) {
      uniVal[0] = "0025";
    }
    else if (in.equals("SquareO")) {
      uniVal[0] = "005b";
    }
    else if (in.equals("SquareC")) {
      uniVal[0] = "005d";
    }
    else if (in.equals("Dha-Yansa")) {
      uniVal[0] = "0daf";
      uniVal[1] = "0dca";
      uniVal[2] = "200d";
      uniVal[3] = "0dba";
    }
    else if (in.equals("Ne")) {
      uniVal[0] = "0dd9";
      uniVal[1] = "0db1";
    }
    else if (in.equals("Nee")) {
      uniVal[0] = "0dd9";
      uniVal[1] = "0db1";
      uniVal[2] = "0dca";
    }
    else if (in.equals("Nna")) {

      uniVal[0] = "0db1";
      uniVal[1] = "0dca";
      uniVal[2] = "0db1";
    }
    else if (in.equals("Ra-Na-M")) {

      uniVal[0] = "0dbb";
      uniVal[1] = "0dab";

    }
    else if (in.equals("CBracketC")) {
      uniVal[0] = "007c";

    }
    else if (in.equals("CBracketO")) {
      uniVal[0] = "007b";

    }
    else if (in.equals("Multi")) {
      uniVal[0] = "0058";

    }
    else if (in.equals("Aee")) {
      uniVal[0] = "0d88";

    }
    else if (in.equals("Baa")) {
      uniVal[0] = "0db6";
      uniVal[1] = "0dcf";

    }
    else if (in.equals("Ba-Alahal")) {
      uniVal[0] = "0db6";
      uniVal[1] = "0dcf";
      uniVal[2] = "0dca";

    }
    else if (in.equals("Be")) {
      uniVal[0] = "0db6";
      uniVal[1] = "0dd0";

    }
    else if (in.equals("Bee")) {
      uniVal[0] = "0db6";
      uniVal[1] = "0dd1";

    }
    else if (in.equals("Daa")) {
      uniVal[0] = "0da9";
      uniVal[1] = "0dcf";

    }
    else if (in.equals("Da-Alahal")) {
      uniVal[0] = "0da9";
      uniVal[1] = "0dcf";
      uniVal[2] = "0dca";

    }
    else if (in.equals("De")) {
      uniVal[0] = "0da9";
      uniVal[1] = "0dd0";

    }
    else if (in.equals("Dee")) {
      uniVal[0] = "0da9";
      uniVal[1] = "0dd1";

    }

    else if (in.equals("Dha-Alahal")) {
      uniVal[0] = "0daf";
      uniVal[1] = "0dcf";
      uniVal[2] = "0dca";

    }
    else if (in.equals("gaa")) {
      uniVal[0] = "0d9c";
      uniVal[1] = "0dcf";

    }
    else if (in.equals("Ga-Alahal")) {
      uniVal[0] = "0d9c";
      uniVal[1] = "0dcf";
      uniVal[2] = "0dca";

    }
    else if (in.equals("Ge")) {
      uniVal[0] = "0d9c";
      uniVal[1] = "0dd0";

    }
    else if (in.equals("Gee")) {
      uniVal[0] = "0d9c";
      uniVal[1] = "0dd1";

    }
    else if (in.equals("Haa")) {
      uniVal[0] = "0dc4";
      uniVal[1] = "0dcf";

    }
    else if (in.equals("Ha-Alahal")) {
      uniVal[0] = "0dc4";
      uniVal[1] = "0dcf";
      uniVal[2] = "0dca";

    }
    else if (in.equals("He")) {
      uniVal[0] = "0dc4";
      uniVal[1] = "0dd0";

    }
    else if (in.equals("Hee")) {
      uniVal[0] = "0dc4";
      uniVal[1] = "0dd1";

    }
    else if (in.equals("Kaa")) {
      uniVal[0] = "0d9a";
      uniVal[1] = "0dcf";

    }
    else if (in.equals("Ka-Alahal")) {
      uniVal[0] = "0d9a";
      uniVal[1] = "0dcf";
      uniVal[2] = "0dca";

    }
    else if (in.equals("Ke")) {
      uniVal[0] = "0d9a";
      uniVal[1] = "0dd0";

    }
    else if (in.equals("Kee")) {
      uniVal[0] = "0d9a";
      uniVal[1] = "0dd1";

    }

    else if (in.equals("Laa")) {
      uniVal[0] = "0d8d";
      uniVal[1] = "0dcf";

    }
    else if (in.equals("La-Alahal")) {
      uniVal[0] = "0d8d";
      uniVal[1] = "0dcf";
      uniVal[2] = "0dca";

    }
    else if (in.equals("Le")) {
      uniVal[0] = "0d8d";
      uniVal[1] = "0dd0";

    }
    else if (in.equals("Lee")) {
      uniVal[0] = "0d8d";
      uniVal[1] = "0dd1";

    }
    else if (in.equals("Maa")) {
      uniVal[0] = "0db8";
      uniVal[1] = "0dcf";

    }
    else if (in.equals("Ma-Alahal")) {
      uniVal[0] = "0db8";
      uniVal[1] = "0dcf";
      uniVal[2] = "0dca";

    }
    else if (in.equals("Me")) {
      uniVal[0] = "0db8";
      uniVal[1] = "0dd0";

    }
    else if (in.equals("Mee")) {
      uniVal[0] = "0db8";
      uniVal[1] = "0dd1";

    }
    else if (in.equals("Naa")) {
      uniVal[0] = "0db1";
      uniVal[1] = "0dcf";

    }
    else if (in.equals("Na-Alahal")) {
      uniVal[0] = "0db1";
      uniVal[1] = "0dcf";
      uniVal[2] = "0dca";

    }
    else if (in.equals("Ne")) {
      uniVal[0] = "0db1";
      uniVal[1] = "0dd0";

    }
    else if (in.equals("Nee")) {
      uniVal[0] = "0db1";
      uniVal[1] = "0dd1";

    }
    else if (in.equals("Pe")) {
      uniVal[0] = "0db4";
      uniVal[1] = "0dd0";

    }
    else if (in.equals("Pee")) {
      uniVal[0] = "0db4";
      uniVal[1] = "0dd1";

    }
    else if (in.equals("Raa")) {
      uniVal[0] = "0dbb";
      uniVal[1] = "0dcf";

    }
    else if (in.equals("Ra-Alahal")) {
      uniVal[0] = "0dbb";
      uniVal[1] = "0dcf";
      uniVal[2] = "0dca";

    }
    else if (in.equals("Saa")) {
      uniVal[0] = "0dc3";
      uniVal[1] = "0dcf";

    }
    else if (in.equals("Sa-Alahal")) {
      uniVal[0] = "0dc3";
      uniVal[1] = "0dcf";
      uniVal[2] = "0dca";

    }
    else if (in.equals("Se")) {
      uniVal[0] = "0dc3";
      uniVal[1] = "0dd0";

    }
    else if (in.equals("See")) {
      uniVal[0] = "0dc3";
      uniVal[1] = "0dd1";

    }
    else if (in.equals("Taa")) {
      uniVal[0] = "0da7";
      uniVal[1] = "0dcf";

    }
    else if (in.equals("Ta-Alahal")) {
      uniVal[0] = "0da7";
      uniVal[1] = "0dcf";
      uniVal[2] = "0dca";

    }
    else if (in.equals("Te")) {
      uniVal[0] = "0da7";
      uniVal[1] = "0dd0";

    }
    else if (in.equals("Tee")) {
      uniVal[0] = "0da7";
      uniVal[1] = "0dd1";

    }
    else if (in.equals("Thaa")) {
      uniVal[0] = "0dad";
      uniVal[1] = "0dcf";

    }
    else if (in.equals("Tha-Alahal")) {
      uniVal[0] = "0dad";
      uniVal[1] = "0dcf";
      uniVal[2] = "0dca";

    }
    else if (in.equals("The")) {
      uniVal[0] = "0dad";
      uniVal[1] = "0dd0";

    }
    else if (in.equals("Thee")) {
      uniVal[0] = "0dad";
      uniVal[1] = "0dd1";

    }
    else if (in.equals("Vaa")) {
      uniVal[0] = "0dc0";
      uniVal[1] = "0dcf";

    }
    else if (in.equals("Va-Alahal")) {
      uniVal[0] = "0dc0";
      uniVal[1] = "0dcf";
      uniVal[2] = "0dca";

    }
    else if (in.equals("Ve")) {
      uniVal[0] = "0dc0";
      uniVal[1] = "0dd0";

    }
    else if (in.equals("Vee")) {
      uniVal[0] = "0dc0";
      uniVal[1] = "0dd1";

    }
    else if (in.equals("Yaa")) {
          uniVal[0] = "0dba";
          uniVal[1] = "0dcf";

        }
        else if (in.equals("Ya-Alahal")) {
          uniVal[0] = "0dba";
          uniVal[1] = "0dcf";
          uniVal[2] = "0dca";

        }
        else if (in.equals("Ye")) {
          uniVal[0] = "0dba";
          uniVal[1] = "0dd0";

        }
        else if (in.equals("Yee")) {
          uniVal[0] = "0dba";
          uniVal[1] = "0dd1";

        }

    else
      uniVal[0] = "003f";
    return uniVal;

  }

  public int[] removeNoise1(int imageData[], int h, int w) {
    int newImage[] = new int[imageData.length];
    for (int k = 0; k < newImage.length; k++)
      newImage[k] = imageData[k];
    for (int y = 1; y < h - 1; y++) {
      for (int x = y * w + 1; x < (y + 1) * w - 1; x++) {
        if ( (imageData[x] & 0xff) < 160) {
          if ( ( ( (imageData[x + 1] & 0xff) >= 160) &&
                ( (imageData[x - 1] & 0xff) >= 160)) ||
              ( ( (imageData[x + w] & 0xff) >= 160) &&
               ( (imageData[x - w] & 0xff) >= 160))) {
            newImage[x] = (255 << 24 | 255 << 16 | 255 << 8 | 255);

          }
        }
      }
    }
    return newImage;
  }

  public int[] removeNoise2(int imageData[], int h, int w) {
    int newImage[] = new int[imageData.length];
    for (int k = 0; k < newImage.length; k++)
      newImage[k] = imageData[k];
    for (int y = 1; y < h - 2; y++) {
      for (int x = y * w + 1; x < (y + 1) * w - 2; x++) {
        if ( ( (imageData[x] & 0xff) < 160) ||
            ( (imageData[x + 1] & 0xff) < 160) ||
            ( (imageData[x + w] & 0xff) < 160) ||
            ( (imageData[x + w + 1] & 0xff) < 160)) {
          if ( ( (imageData[x - w - 1] & 0xff) >= 160) &&
              ( (imageData[x - w] & 0xff) >= 160) &&
              ( (imageData[x + w] & 0xff) >= 160) &&
              ( (imageData[x + w + 1] & 0xff) >= 160) &&
              ( (imageData[x + w + 2] & 0xff) >= 160) &&
              ( (imageData[x - 1] & 0xff) >= 160) &&
              ( (imageData[x + 2] & 0xff) >= 160) &&
              ( (imageData[x + w - 1] & 0xff) >= 160) &&
              ( (imageData[x + w + 2] & 0xff) >= 160) &&
              ( (imageData[x + 2 * w - 1] & 0xff) >= 160) &&
              ( (imageData[x + 2 * w] & 0xff) >= 160) &&
              ( (imageData[x + 2 * w + 1] & 0xff) >= 160) &&
              ( (imageData[x + 2 * w + 2] & 0xff) >= 160)) {
            newImage[x] = (255 << 24 | 255 << 16 | 255 << 8 | 255);
            newImage[x + 1] = (255 << 24 | 255 << 16 | 255 << 8 | 255);
            newImage[x + w] = (255 << 24 | 255 << 16 | 255 << 8 | 255);
            newImage[x + w + 1] = (255 << 24 | 255 << 16 | 255 << 8 | 255);

          }
        }
      }
    }
    return newImage;
  }

  public static String charRec(String source, String font, String target) {
    String status = "";
    try {
      RandomAccessFile outputResult = new RandomAccessFile(target, "rw");
      ArrayList arrayList = new ArrayList();

      try {
        Calendar calendar = Calendar.getInstance();
        Date ds = calendar.getTime();
        int stTime = ds.getMinutes() * 60 + ds.getSeconds();
        String sourcePath = source;

        DocumentRec doc = new DocumentRec(sourcePath);
        int noiselessData[] = doc.removeNoise1(doc.removeNoise2(doc.getData(),
            doc.getHeight(),
            doc.getWidth()), doc.getHeight(), doc.getWidth());
        //int noiselessData[] = doc.removeNoise1(doc.getData(), doc.getHeight(),
        //    doc.getWidth());
        doc.setNewData(noiselessData);
        doc.calculateLineCount();
        doc.averageLineHeight();
        doc.setLineArray();
        byte enter[] = {
            13, 10};
        int lineCount = doc.getLineCount();

        int totalChars = 0;
        int paraInfo[] = doc.paraInfo();

        for (int k = 0; k < lineCount; k++) {
          Line temp_line = doc.getLine(k);
          temp_line.extractPixelValues(doc.getData());
          temp_line.averageCharLength();
          temp_line.setCharArray();
          int lineStart[] = temp_line.getCharStartInfo();
          int lineEnd[] = temp_line.getCharEndInfo();
          int charCount = temp_line.calculateCharCount();
          totalChars = totalChars + charCount;
          for (int y = 0; y < charCount; y++) {
            Char temp_char = temp_line.getCharArray()[y];
            temp_char.extractLinePixelValues(temp_line.getData(),
                                             temp_line.getLineWidth());
            ImageProducer m_char_image1 = new MemoryImageSource(temp_char.
                getWidth(), temp_char.getHeight(), temp_char.getCharData(), 0,
                temp_char.getWidth());
            Image char_image1 = Toolkit.getDefaultToolkit().createImage(
                m_char_image1);
            int com_data[] = new int[temp_char.getWidth() * temp_char.getHeight()];
            PixelGrabber com_grabber = new PixelGrabber(char_image1, 0, 0,
                char_image1.getWidth(null), char_image1.getHeight(null),
                com_data,
                0, char_image1.getWidth(null));
            com_grabber.grabPixels();
            Char new_char = doc.getCompactImage(com_data, temp_char.getHeight(),
                                                temp_char.getWidth());
            ImageProducer m_char_image = new MemoryImageSource(new_char.
                getWidth(),
                new_char.getHeight(), new_char.getCharData(), 0,
                new_char.getWidth());
            Image char_image = Toolkit.getDefaultToolkit().createImage(
                m_char_image).getScaledInstance(25, 25, 2);
            int compressedImage[] = new int[625]; //int[char_image.getWidth(null)*char_image.getHeight(null)];
            PixelGrabber compressed_grabber = new PixelGrabber(char_image, 0, 0,
                char_image.getWidth(null), char_image.getHeight(null),
                compressedImage, 0, char_image.getWidth(null));
            compressed_grabber.grabPixels();
            for (int z = 0; z < compressedImage.length; z++) {
              int grey = (int) (compressedImage[z] & 0xff);
              if (grey > 160) {
                compressedImage[z] = 1;
              }
              else {
                compressedImage[z] = 0;
              }
            }
            String currentLetter = doc.knn(compressedImage, k, y, font);
            String unicodeValues[] = doc.unicodeValue(currentLetter.substring(3,
                (currentLetter.lastIndexOf('.'))));
            for (int x = 0; x < unicodeValues.length; x++) {
              if (unicodeValues[x] == null) {
                break;

              }
              arrayList.add(unicodeValues[x]);
            }
            if (y + 1 >= charCount)
              continue;
            int difference = (lineStart[y + 1] - lineEnd[y]);
            if ( (difference > 10) && (y != 0))
              arrayList.add("0020");
          }
          arrayList.add("0020");
          for (int g = 0; g < paraInfo.length; g++) {
            if (paraInfo[g] == 0)
              break;
            if (paraInfo[g] == k) {
              arrayList.add("000d");
              arrayList.add("000a");
              arrayList.add("000d");
              arrayList.add("000a");
            }

          }

        }
        calendar = Calendar.getInstance();
        ds = calendar.getTime();
        int enTime = ds.getMinutes() * 60 + ds.getSeconds();
        int duration = enTime - stTime;
        System.out.println("Total chracters " + totalChars);
        System.out.println("minutes " + (duration / 60) + " seconds " +
                           (duration % 60));
        OCRN.Normalizer(arrayList, outputResult);
        outputResult.close();
        status="OK";
      }
      catch (ArithmeticException e) {
        System.out.println("Too much noise");
        System.out.println(e);
        status="NOISE";
      }
      finally {
        OCRN.Normalizer(arrayList, outputResult);
        outputResult.close();
      }
    }
    catch (Exception ee) {status="NOISE";}
    return status;
  }

}